@@ -11,7 +11,7 @@ package UTF8::R2;
 use 5.00503;    # Universal Consensus 1998 for primetools
 # use 5.008001; # Lancaster Consensus 2013 for toolchains
 
-$VERSION = '0.08';
+$VERSION = '0.09';
 $VERSION = $VERSION;
 
 use strict;
@@ -626,12 +626,45 @@ sub UTF8::R2::substr ($$;$$) %s {
 }
 END
 
+#---------------------------------------------------------------------
+# tr/A-C/1-3/ for UTF-8 codepoint
+sub _list_all_ASCII_by_hyphen {
+    my @hyphened = @_;
+    my @list_all = ();
+    for (my $i=0; $i <= $#hyphened; ) {
+        if (
+            ($i+1 < $#hyphened)      and
+            ($hyphened[$i+1] eq '-') and
+        1) {
+            if (0) { }
+            elsif ($hyphened[$i+0] !~ m/\A [\x00-\x7F] \z/oxms) {
+                Carp::confess sprintf(qq{@{[__FILE__]}: "$hyphened[$i+0]-$hyphened[$i+2]" in tr/// is not ASCII});
+            }
+            elsif ($hyphened[$i+2] !~ m/\A [\x00-\x7F] \z/oxms) {
+                Carp::confess sprintf(qq{@{[__FILE__]}: "$hyphened[$i+0]-$hyphened[$i+2]" in tr/// is not ASCII});
+            }
+            elsif ($hyphened[$i+0] gt $hyphened[$i+2]) {
+                Carp::confess sprintf(qq{@{[__FILE__]}: "$hyphened[$i+0]-$hyphened[$i+2]" in tr/// is not "$hyphened[$i+0]" le "$hyphened[$i+2]"});
+            }
+            else {
+                push @list_all, ($hyphened[$i+0] .. $hyphened[$i+2]);
+                $i += 3;
+            }
+        }
+        else {
+            push @list_all, $hyphened[$i];
+            $i++;
+        }
+    }
+    return @list_all;
+}
+
 #---------------------------------------------------------------------
 # tr/// for UTF-8 codepoint string
 sub UTF8::R2::tr ($$$;$) {
     my @x           = $_[0] =~ /\G$x/g;
-    my @search      = $_[1] =~ /\G$x/g;
-    my @replacement = $_[2] =~ /\G$x/g;
+    my @search      = _list_all_ASCII_by_hyphen($_[1] =~ /\G$x/g);
+    my @replacement = _list_all_ASCII_by_hyphen($_[2] =~ /\G$x/g);
     my %modifier    = (defined $_[3]) ? (map { $_ => 1 } CORE::split //, $_[3]) : ();
 
     my %tr = ();
@@ -851,7 +884,7 @@ UTF8::R2 - makes UTF-8 scripting easy for enterprise use or LTS
     $result = UTF8::R2::rindex($_, 'ABC', 5)
     @result = UTF8::R2::split(qr/$utf8regex/, $_, 3)
     $result = UTF8::R2::substr($_, 0, 5)
-    $result = UTF8::R2::tr($_, 'ABC', 'XYZ', 'cdsr')
+    $result = UTF8::R2::tr($_, 'A-C', 'X-Z', 'cdsr')
     $result = UTF8::R2::uc($_)
     $result = UTF8::R2::ucfirst($_)
 
@@ -921,7 +954,7 @@ UTF-8 codepoint semantics is provided by the new subroutine name.
   substr                  UTF8::R2::substr($_, 0, 5)                 substr() is compatible and usually useful
                                                                      :lvalue feature needs perl 5.014 or later
   ------------------------------------------------------------------------------------------------------------------------------------------
-  tr/// or y///           UTF8::R2::tr($_, 'ABC', 'XYZ', 'cdsr')     not support range of codepoint(like a "tr/A-Z/a-z/")
+  tr/// or y///           UTF8::R2::tr($_, 'A-C', 'X-Z', 'cdsr')     range of codepoint by hyphen supports ASCII only
   ------------------------------------------------------------------------------------------------------------------------------------------
   uc                      UTF8::R2::uc($_)                           works as tr/a-z/A-Z/, universally
   ------------------------------------------------------------------------------------------------------------------------------------------
